@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -31,7 +31,7 @@ import socket
 import stat
 
 from .constants import DEFAULT_LANG, EXTENDED_DATA_STDERR
-from .misc import Error, ProtocolError, Record
+from .misc import Error, ProtocolError, Record, open_file
 from .stream import SSHClientStreamSession, SSHServerStreamSession
 from .stream import SSHReader, SSHWriter
 
@@ -454,7 +454,7 @@ class ProcessError(Error):
                       process to execute (if any)
          subsystem    The subsystem the client requested the  `str` or `None`
                       process to open (if any)
-         exit_status  The exit status returned, or -1 if an   `int`
+         exit_status  The exit status returned, or -1 if an   `int` or `None`
                       exit signal is sent
          exit_signal  The exit signal sent (if any) in the    `tuple` or `None`
                       form of a tuple containing the signal
@@ -462,7 +462,7 @@ class ProcessError(Error):
                       occurred, a message associated with the
                       signal, and the language the message
                       was in
-         returncode   The exit status returned, or negative   `int`
+         returncode   The exit status returned, or negative   `int` or `None`
                       of the signal number when an exit
                       signal is sent
          stdout       The output sent by the process to       `str` or `bytes`
@@ -474,7 +474,8 @@ class ProcessError(Error):
     """
 
     def __init__(self, env, command, subsystem, exit_status,
-                 exit_signal, returncode, stdout, stderr):
+                 exit_signal, returncode, stdout, stderr,
+                 reason='', lang=DEFAULT_LANG):
         self.env = env
         self.command = command
         self.subsystem = subsystem
@@ -489,14 +490,30 @@ class ProcessError(Error):
             reason = 'Process exited with signal %s%s%s' % \
                 (signal, ': ' + msg if msg else '',
                  ' (core dumped)' if core_dumped else '')
-        else:
+        elif exit_status:
             reason = 'Process exited with non-zero exit status %s' % \
                 exit_status
-            lang = DEFAULT_LANG
 
         super().__init__(exit_status, reason, lang)
 
 
+# pylint: disable=redefined-builtin
+class TimeoutError(ProcessError, asyncio.TimeoutError):
+    """SSH Process timeout error
+
+       This exception is raised when a timeout occurs when calling the
+       :meth:`wait <SSHClientProcess.wait>` method on :class:`SSHClientProcess`
+       or the :meth:`run <SSHClientConnection.run>` method on
+       :class:`SSHClientConnection`. It is a subclass of :class:`ProcessError`
+       and contains all of the fields documented there, including any output
+       received on stdout and stderr prior to when the timeout occurred. It
+       is also a subclass of :class:`asyncio.TimeoutError`, for code that
+       might be expecting that.
+
+    """
+# pylint: enable=redefined-builtin
+
+
 class SSHCompletedProcess(Record):
     """Results from running an SSH process
 
@@ -660,9 +677,9 @@ class SSHProcess:
                                    self._encoding, self._errors)
         else:
             if isinstance(source, str):
-                file = open(source, 'rb', buffering=bufsize)
+                file = open_file(source, 'rb', buffering=bufsize)
             elif isinstance(source, PurePath):
-                file = open(str(source), 'rb', buffering=bufsize)
+                file = open_file(str(source), 'rb', buffering=bufsize)
             elif isinstance(source, int):
                 file = os.fdopen(source, 'rb', buffering=bufsize)
             elif isinstance(source, socket.socket):
@@ -714,9 +731,9 @@ class SSHProcess:
             writer = _StreamWriter(target, self._encoding, self._errors)
         else:
             if isinstance(target, str):
-                file = open(target, 'wb', buffering=bufsize)
+                file = open_file(target, 'wb', buffering=bufsize)
             elif isinstance(target, PurePath):
-                file = open(str(target), 'wb', buffering=bufsize)
+                file = open_file(str(target), 'wb', buffering=bufsize)
             elif isinstance(target, int):
                 file = os.fdopen(target, 'wb', buffering=bufsize)
             elif isinstance(target, socket.socket):
@@ -1168,7 +1185,7 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
 
         self._chan.kill()
 
-    async def wait(self, check=False):
+    async def wait(self, check=False, timeout=None):
         """Wait for process to exit
 
            This method is a coroutine which waits for the process to
@@ -1180,18 +1197,37 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
            status from the process with trigger the :exc:`ProcessError`
            exception to be raised.
 
+           If a timeout is specified and it expires before the process
+           exits, the :exc:`TimeoutError` exception will be raised. By
+           default, no timeout is set and this call will wait indefinitely.
+
            :param check:
                Whether or not to raise an error on non-zero exit status
+           :param timeout:
+               Amount of time in seconds to wait for process to exit, or
+               `None` to wait indefinitely
            :type check: `bool`
+           :type timeout: `int`, `float`, or `None`
 
            :returns: :class:`SSHCompletedProcess`
 
-           :raises: :exc:`ProcessError` if check is set to `True`
-                    and the process returns a non-zero exit status
+           :raises: | :exc:`ProcessError` if check is set to `True`
+                      and the process returns a non-zero exit status
+                    | :exc:`TimeoutError` if the timeout expires
+                      before the process exits
 
         """
 
-        stdout_data, stderr_data = await self.communicate()
+        try:
+            stdout_data, stderr_data = \
+                await asyncio.wait_for(self.communicate(), timeout)
+        except asyncio.TimeoutError:
+            stdout_data, stderr_data = self.collect_output()
+
+            raise TimeoutError(self.env, self.command, self.subsystem,
+                               self.exit_status, self.exit_signal,
+                               self.returncode, stdout_data,
+                               stderr_data) from None
 
         if check and self.exit_status:
             raise ProcessError(self.env, self.command, self.subsystem,
