@@ -783,11 +783,10 @@ class SSHServer:
 
            :returns: One of the following:
 
-                     * An :class:`SSHListener` object or a coroutine
-                       which returns an :class:`SSHListener` or `False`
-                       if the listener can't be opened
+                     * An :class:`SSHListener` object
                      * `True` to set up standard port forwarding
                      * `False` to reject the request
+                     * A coroutine object which returns one of the above
 
         """
 
