@@ -21,6 +21,7 @@
 """Unit tests for AsyncSSH channel API"""
 
 import asyncio
+import os
 import tempfile
 from signal import SIGINT
 
@@ -234,6 +235,7 @@ class _ChannelServer(Server):
 
             if agent_path:
                 async with asyncssh.connect_agent(agent_path) as agent:
+                    await asyncio.sleep(0.1)
                     stdout.write(str(len((await agent.get_keys()))) + '\n')
             else:
                 stdout.channel.exit(1)
@@ -714,7 +716,7 @@ class _TestChannel(ServerTestCase):
             chan, _ = await _create_session(conn)
 
             with self.assertRaises(OSError):
-                await chan.create(None, None, None, {}, None, None,
+                await chan.create(None, None, None, {}, False, None, None,
                                   None, False, None, None, False, False)
 
             chan.close()
@@ -943,6 +945,19 @@ class _TestChannel(ServerTestCase):
             result = ''.join(session.recv_buf[None])
             self.assertEqual(result, "('ansi', (80, 24, 480, 240), 9600)\r\n")
 
+    @asynctest
+    async def test_pty_without_term_type(self):
+        """Test requesting a PTY without setting the terminal type"""
+
+        async with self.connect() as conn:
+            chan, session = await _create_session(conn, 'term',
+                                                  request_pty='force')
+
+            await chan.wait_closed()
+
+            result = ''.join(session.recv_buf[None])
+            self.assertEqual(result, "('', (0, 0, 0, 0), None)\n")
+
     @asynctest
     async def test_invalid_terminal_size(self):
         """Test sending invalid terminal size"""
@@ -1025,7 +1040,7 @@ class _TestChannel(ServerTestCase):
 
     @asynctest
     async def test_env(self):
-        """Test sending environment"""
+        """Test setting environment"""
 
         async with self.connect() as conn:
             chan, session = await _create_session(conn, 'env',
@@ -1036,6 +1051,62 @@ class _TestChannel(ServerTestCase):
             result = ''.join(session.recv_buf[None])
             self.assertEqual(result, 'test\n')
 
+    @asynctest
+    async def test_env_list(self):
+        """Test setting environment using a list of name=value strings"""
+
+        async with self.connect() as conn:
+            chan, session = await _create_session(conn, 'env',
+                                                  env=['TEST=test'])
+
+            await chan.wait_closed()
+
+            result = ''.join(session.recv_buf[None])
+            self.assertEqual(result, 'test\n')
+
+    @asynctest
+    async def test_invalid_env_list(self):
+        """Test setting environment using an invalid string"""
+
+        with self.assertRaises(ValueError):
+            async with self.connect() as conn:
+                await _create_session(conn, 'env', env=['XXX'])
+
+    @asynctest
+    async def test_send_env(self):
+        """Test sending local environment"""
+
+        async with self.connect() as conn:
+            try:
+                os.environ['TEST'] = 'test'
+                chan, session = await _create_session(conn, 'env',
+                                                      send_env=['TEST'])
+            finally:
+                del os.environ['TEST']
+
+            await chan.wait_closed()
+
+            result = ''.join(session.recv_buf[None])
+            self.assertEqual(result, 'test\n')
+
+    @asynctest
+    async def test_mixed_env(self):
+        """Test sending a mix of local environment and new values"""
+
+        async with self.connect() as conn:
+            try:
+                os.environ['TEST'] = '1'
+                chan, session = await _create_session(conn, 'env',
+                                                      env={'TEST': 2},
+                                                      send_env='TEST')
+            finally:
+                del os.environ['TEST']
+
+            await chan.wait_closed()
+
+            result = ''.join(session.recv_buf[None])
+            self.assertEqual(result, '2\n')
+
     @asynctest
     async def test_invalid_env(self):
         """Test sending invalid environment"""
@@ -1418,3 +1489,67 @@ class _TestChannel(ServerTestCase):
 
             await chan.wait_closed()
             self.assertEqual(session.exit_status, 255)
+
+
+class _TestChannelNoPTY(ServerTestCase):
+    """Unit tests for AsyncSSH channel module with PTYs disallowed"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server for the tests to use"""
+
+        return (await cls.create_server(
+            _ChannelServer, authorized_client_keys='authorized_keys',
+            allow_pty=False))
+
+    @asynctest
+    async def test_shell_pty(self):
+        """Test starting a shell that request a PTY"""
+
+        async with self.connect() as conn:
+            with self.assertRaises(asyncssh.ChannelOpenError):
+                await conn.run(term_type='ansi')
+
+    @asynctest
+    async def test_shell_no_pty(self):
+        """Test starting a shell that doesn't request a PTY"""
+
+        async with self.connect() as conn:
+            await conn.run(request_pty=False, stdin=asyncssh.DEVNULL)
+
+    @asynctest
+    async def test_exec_pty(self):
+        """Test execution of a remote command that requests a PTY"""
+
+        async with self.connect() as conn:
+            with self.assertRaises(asyncssh.ChannelOpenError):
+                await conn.run('echo', request_pty='force')
+
+    @asynctest
+    async def test_exec_no_pty(self):
+        """Test execution of a remote command that doesn't request a PTY"""
+
+        async with self.connect() as conn:
+            await conn.run('echo', term_type='ansi', request_pty='auto',
+                           stdin=asyncssh.DEVNULL)
+
+
+class _TestChannelNoAgentForwarding(ServerTestCase):
+    """Unit tests for channel module with agent forwarding disallowed"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server with agent forwarding disabled"""
+
+        return (await cls.create_server(
+            _ChannelServer, authorized_client_keys='authorized_keys',
+            agent_forwarding=False))
+
+    @asynctest
+    async def test_agent_forwarding_disallowed(self):
+        """Test starting a shell that request a PTY"""
+
+        async with self.connect(agent_forwarding=True) as conn:
+            result = await conn.run('agent')
+
+        self.assertEqual(result.exit_status, 1)
