@@ -98,6 +98,8 @@ class _EchoPortListener(asyncssh.SSHListener):
     """A TCP listener which opens a connection that echoes data"""
 
     def __init__(self, conn):
+        super().__init__()
+
         self._conn = conn
 
         conn.create_task(self._open_connection())
@@ -120,6 +122,8 @@ class _EchoPathListener(asyncssh.SSHListener):
     """A UNIX domain listener which opens a connection that echoes data"""
 
     def __init__(self, conn):
+        super().__init__()
+
         self._conn = conn
 
         conn.create_task(self._open_connection())
@@ -167,6 +171,18 @@ class _TCPConnectionServer(Server):
             return listen_host != 'fail'
 
 
+class _TCPAsyncConnectionServer(_TCPConnectionServer):
+    """Server for testing async direct and forwarded TCP connections"""
+
+    async def server_requested(self, listen_host, listen_port):
+        """Handle a request to create a new socket listener"""
+
+        if listen_host == 'open':
+            return _EchoPortListener(self._conn)
+        else:
+            return listen_host != 'fail'
+
+
 class _UNIXConnectionServer(Server):
     """Server for testing direct and forwarded UNIX domain connections"""
 
@@ -277,6 +293,23 @@ class _TestTCPForwarding(_CheckForwarding):
                                         self._server_port) as conn2:
                 await self._check_connection(conn2)
 
+    @asynctest
+    async def test_ssh_connect_tunnel_string(self):
+        """Test connecting a tunneled SSH connection via string"""
+
+        async with self.connect(tunnel='%s:%d' % (self._server_addr,
+                                                  self._server_port)) as conn:
+            await self._check_connection(conn)
+
+    @asynctest
+    async def test_ssh_connect_tunnel_string_failed(self):
+        """Test failed connection on a tunneled SSH connection via string"""
+
+        with self.assertRaises(asyncssh.ChannelOpenError):
+            await asyncssh.connect('0.0.0.1',
+                                   tunnel='%s:%d' % (self._server_addr,
+                                                     self._server_port))
+
     @asynctest
     async def test_ssh_connect_reverse_tunnel(self):
         """Test creating a tunneled reverse direction SSH connection"""
@@ -287,8 +320,7 @@ class _TestTCPForwarding(_CheckForwarding):
         async with self.connect() as conn:
             async with conn.connect_reverse_ssh('127.0.0.1', listen_port,
                                                 server_factory=Server,
-                                                server_host_keys=['skey'],
-                                                gss_host=None):
+                                                server_host_keys=['skey']):
                 pass
 
         server2.close()
@@ -300,12 +332,48 @@ class _TestTCPForwarding(_CheckForwarding):
 
         async with self.connect() as conn:
             async with conn.listen_ssh(port=0, server_factory=Server,
-                                       server_host_keys=['skey'],
-                                       gss_host=None) as server2:
+                                       server_host_keys=['skey']) as server2:
                 listen_port = server2.get_port()
 
                 async with asyncssh.connect('127.0.0.1', listen_port,
-                                            gss_host=None,
+                                            known_hosts=(['skey.pub'], [], [])):
+                    pass
+
+    @asynctest
+    async def test_ssh_listen_tunnel_string(self):
+        """Test opening a tunneled SSH listener via string"""
+
+        async with self.listen(tunnel='ckey@%s:%d' % (self._server_addr,
+                                                      self._server_port),
+                               server_factory=Server,
+                               server_host_keys=['skey']) as server:
+            listen_port = server.get_port()
+
+            async with asyncssh.connect('127.0.0.1', listen_port,
+                                        known_hosts=(['skey.pub'], [], [])):
+                pass
+
+    @asynctest
+    async def test_ssh_listen_tunnel_string_failed(self):
+        """Test open failure on a tunneled SSH listener via string"""
+
+        with self.assertRaises(asyncssh.ChannelListenError):
+            await asyncssh.listen('0.0.0.1',
+                                  tunnel='%s:%d' % (self._server_addr,
+                                                    self._server_port),
+                                  server_factory=Server,
+                                  server_host_keys=['skey'])
+
+    @asynctest
+    async def test_ssh_listen_tunnel_default_port(self):
+        """Test opening a tunneled SSH listener via string without port"""
+
+        with patch('asyncssh.connection.DEFAULT_PORT', self._server_port):
+            async with self.listen(tunnel='localhost', server_factory=Server,
+                                   server_host_keys=['skey']) as server:
+                listen_port = server.get_port()
+
+                async with asyncssh.connect('127.0.0.1', listen_port,
                                             known_hosts=(['skey.pub'], [], [])):
                     pass
 
@@ -314,14 +382,13 @@ class _TestTCPForwarding(_CheckForwarding):
         """Test creating a tunneled reverse direction SSH connection"""
 
         async with self.connect() as conn:
-            async with conn.listen_reverse_ssh(port=0, gss_host=None,
+            async with conn.listen_reverse_ssh(port=0,
                                                known_hosts=(['skey.pub'],
                                                             [], [])) as server2:
                 listen_port = server2.get_port()
 
                 async with asyncssh.connect_reverse('127.0.0.1', listen_port,
                                                     server_factory=Server,
-                                                    gss_host=None,
                                                     server_host_keys=['skey']):
                     pass
 
@@ -647,6 +714,17 @@ class _TestTCPForwarding(_CheckForwarding):
                 await listener.wait_closed()
 
 
+class _TestAsyncTCPForwarding(_TestTCPForwarding):
+    """Unit tests for AsyncSSH TCP connection forwarding with async return"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server which supports TCP connection forwarding"""
+
+        return await cls.create_server(
+            _TCPAsyncConnectionServer, authorized_client_keys='authorized_keys')
+
+
 @unittest.skipIf(sys.platform == 'win32',
                  'skip UNIX domain socket tests on Windows')
 class _TestUNIXForwarding(_CheckForwarding):
